#!/usr/bin/perl -w
#===============================================================================
# Name:        PerlStock.pl
# Description: sequencer for daily trading routine
# Copyright:   (c) 2015, by Felix Bertram
#===============================================================================

use strict;
use warnings;

use Data::Dumper;
use DateTime;
use Win32::TieRegistry( Delimiter => "/");
use Win32::GuiTest qw(:ALL);

use Email;
use TradingSystem;

#===============================================================================
# the following values are expected in the Windows registry:
# HKEY_CURRENT_USER/PerlStock
#   --- keys for IB TWS
#   ib_usr   REG_SZ    IB username
#   ib_pws   REG_SZ    IB password
#   ib_exe   REG_SZ    path to TWS shortcut in start menu
#   ib_xml   REG_SZ    path to TWS .audit.xml files
#   --- keys for Multicharts Portfolio Trader
#   mc_prj   REG_SZ    path to .pws file
#   --- keys for emailing feature
#   e_usr    REG_SZ    username for email service
#   e_pwd    REG_SZ    password for email service
#   e_from   REG_SZ    email from field, e.g. John Doe <john.doe@myservice.domain>
#   e_to     REG_SZ    email to field, e.g. Betty Blue <betty.blue@myservice.domain>
#   e_ena    REG_SZ    1 to send emails, 0 to print to console only
#   --- general keys
#   sys_sd   REG_SZ    1 to call shutdown /s, 0 to keep system on
#
my $mystuff = $Registry->{"HKEY_CURRENT_USER/PerlStock"};

# email parameters
my $e_from = $mystuff->{"e_from"};
my $e_usr  = $mystuff->{"e_usr"};
my $e_pwd  = $mystuff->{"e_pwd"};
my $e_to   = $mystuff->{"e_to"};
my $e_ena  = $mystuff->{"e_ena"};

# TWS parameters
my $ib_exe = $mystuff->{"ib_exe"};
my $ib_xml = $mystuff->{"ib_xml"};
my $ib_usr = $mystuff->{"ib_usr"};
my $ib_pwd = $mystuff->{"ib_pwd"};

# MC parameters
my $mc_prj = $mystuff->{"mc_prj"};

# general parameters
my $sys_sd = $mystuff->{"sys_sd"};

#===============================================================================
# Name:        sendMsg
# Description: send message: print to console and/or send via email
sub sendMsg {
	my ($msg) = @_;


	print "$msg";

	if ($e_ena > 0) {
		print "  => also sent by email\n";

		$msg =~ s/\n+$//;
		my $mailer = Email->new(from => $e_from, user => $e_usr, pwd => $e_pwd);
		$mailer->email($e_to, $msg);
	}
}

#===============================================================================
# Name:        getEST
# Description: return time for America/New York
sub getEST {
	my $dt = DateTime->now(time_zone => 'America/New_York');
	$dt =~ m/....-..-..T(..):(..):(..)/;
	my @t = ($1, $2, $3);
	return @t;
}

#-------------------------------------------------------------------------------
# Name:        getSleepMins
# Description: calculate number of minutes until target time
sub getSleepMins {
	my ($hh2, $mm2) = @_;

	my ($hh, $mm) = (getEST());
	my $sleepmins = $hh2 * 60 + $mm2 - ($hh * 60 + $mm);

	return $sleepmins;
}

#-------------------------------------------------------------------------------
# Name:        modSleepMins
# Description: perform modulo operation on sleep mins
sub modSleepMins {
	my ($sleepmins, $modulo) = @_;

	if ($sleepmins > $modulo) {
		$sleepmins = $sleepmins % $modulo;
		$sleepmins = $modulo if ($sleepmins == 0);
	}
	return $sleepmins;
}

#===============================================================================
# Name:        dailySequence
# Description: run through our daily sequence
sub dailySequence {
	my ($hh, $mm, $sleepmins);

	($hh, $mm) = (getEST());
	sendMsg("$hh:$mm: start script\n");

	if (0 && (getSleepMins(17, 0) < 0)) {
		while (($sleepmins = getSleepMins(24, 0)) > 0) {
			$sleepmins = modSleepMins($sleepmins, 15);
			($hh, $mm) = (getEST());
			print "$hh:$mm: wait for $sleepmins minutes\n";

			sleep($sleepmins * 60);
		};
		sleep(5 * 60);
	}

	#--- wait until about 9:00EST, trading begins at 9:30EST
	while (($sleepmins = getSleepMins(9,0)) > 0) {
		$sleepmins = modSleepMins($sleepmins, 15);
		($hh, $mm) = (getEST());
		print "$hh:$mm: wait for $sleepmins minutes\n";

		sleep($sleepmins * 60);
	};

	#--- start trading system
	($hh, $mm) = (getEST());
	sendMsg("$hh:$mm: start trading system\n");

	my $ats = TradingSystem->new(
		ib_exe => $ib_exe, ib_xml => $ib_xml, ib_usr => $ib_usr, ib_pwd => $ib_pwd, 
		mc_prj => $mc_prj);
	$ats->start() if (getSleepMins(17,0) > 0);
#$ats->start(); # debugging only

	#--- watch trading system until 17:00EST, trading closes at 16:00EST
	while (($sleepmins = getSleepMins(17,0)) > 0) {
		$sleepmins = modSleepMins($sleepmins, 5);
		($hh, $mm) = (getEST());
		print "$hh:$mm: wait for $sleepmins minutes\n";

		sleep($sleepmins * 60);

		($hh, $mm) = (getEST());
		my $newMsg  = $ats->getNewMsg();
		foreach my $key (sort(keys(%{$newMsg}))) {
			sendMsg("$hh:$mm: ". $ats->getMsgText($key) ."\n");
		}
	};

	#--- stop trading system
	($hh, $mm) = (getEST());
	sendMsg("$hh:$mm: stop trading system\n");
	sleep(5 * 60);

	$ats->stop($sys_sd);

	#--- end script
	($hh, $mm) = (getEST());
	sendMsg "$hh:$mm: end script\n";
}

#===============================================================================
dailySequence();
1;

#===============================================================================
# end of file
