#===============================================================================
# File:        TradingSystem.pm
# Description: Manage trading system
#              Interactive Brokers' TWS
#              MultiCharts 9
# Copyright:   (c) 2015, by Felix Bertram
#===============================================================================

use strict;
use warnings;

use Data::Dumper;
use XML::LibXML;
use Win32::Job;
use Win32;
use Win32::Process;
use Win32::GuiTest qw(:ALL);

package TradingSystem;

#-------------------------------------------------------------------------------
# Name:        new
# Description: create new TradingSystem object
sub new {
	my ($class, %args) = @_;

	my $self = {
		ib_xml  => $args{ib_xml},
		ib_exe  => $args{ib_exe},
		ib_usr  => $args{ib_usr},
		ib_pwd  => $args{ib_pwd},
		mc_prj  => $args{mc_prj},
		oldmsg  => (),
		newmsg  => (),
	};
	bless($self, $class);

	$self->_scanXmlFiles(1);

	return $self;
}

#-------------------------------------------------------------------------------
# Name:        start
# Description: start trading system: launch TWS and MultiCharts
use Win32;
use Win32::Process;
use Win32::GuiTest qw(:ALL);
sub start {
	my ($self) = @_;

	#--- start IB TWS
	system(1, '"'. $self->{ib_exe} .'"'); 

	#--- wait for login window to come up
	my @ib_wins;
	do {
		sleep(5);
		print "waiting for TWS login to come up...\n";
	} while(scalar(@ib_wins = FindWindowLike(undef, "Login", "")) < 1);

	#--- log into TWS
	sleep(2);
	SetActiveWindow($ib_wins[0]);
	SetForegroundWindow($ib_wins[0]);
	SendKeys($self->{ib_usr});
	SendKeys('{TAB}');
	SendKeys($self->{ib_pwd});
	SendKeys('~');

	#--- wait for TWS to come up
	do {
		sleep(5);
		print "waiting for TWS to come up...\n";
	} while (scalar(FindWindowLike(undef, "U...+ Interactive Brokers")) < 1);

	#--- start MultiCharts
	system(1, '"'. $self->{mc_prj} .'"');

	#--- wait for MultiCharts to come up
	my @mc_wins;
	do {
		sleep(5);
		print "waiting for MultiCharts to come up...\n";
	} while (scalar(@mc_wins = FindWindowLike(undef, "MultiCharts64 Portfolio Trader -", "")) < 1);

	#--- start auto-trading
	sleep(2);
	SetActiveWindow($mc_wins[0]);
	SetForegroundWindow($mc_wins[0]);
	SendKeys('%p'); # open 'Portfolio' menu
	SendKeys('{DOWN}{DOWN}{DOWN}{DOWN}{ENTER}'); # select 'Run Automate Order Execution' 

	#--- wait for auto-trading confirmation to come up
	my @auto_wins;
	do {
		sleep(5);
		print "waiting for trading confirmation to come up...\n";
	} while (scalar(@auto_wins = FindWindowLike(undef, "MultiCharts64", "#32770")) < 1);

	#--- confirm auto-trading
	sleep(2);
	SetActiveWindow($auto_wins[0]);
	SetForegroundWindow($auto_wins[0]);
	SendKeys('y'); # say 'Yes'
}

#-------------------------------------------------------------------------------
# Name:        stop
# Description: stop trading system: shutdown TWS and MultiCharts
sub stop {
	my ($self, $sd) = @_;
	$sd = 0 if (!defined($sd));

	#--- cleanly exit Portfolio Trader

	#--- cleanly exit TWS

	#--- shut down computer
	if ($sd > 0) {
		system(1, "shutdown /s");
	}
}

#-------------------------------------------------------------------------------
# Name:        getNewMsg
# Description: return array of new messages
sub getNewMsg {
	my ($self) = @_;

	$self->_scanXmlFiles();
	return $self->{newmsg};
}

#-------------------------------------------------------------------------------
# Name:        _scanXmlFiles
# Description: scan the XML audit trail files for any changes
sub getMsgText {
	my ($self, $timestamp) = @_;
	return $self->{newmsg}{$timestamp};
}

#-------------------------------------------------------------------------------
# Name:        _scanXmlFiles
# Description: scan the XML audit trail files for any changes
use Data::Dumper;
sub _scanXmlFiles {
	my ($self, $reset) = @_;

	$reset = 0 if (!defined($reset));

	$self->{newmsg} = (); #undef ($self->{newmsg});

	opendir(my $dh, $self->{ib_xml}) || die "ERROR: failed to open directory ". $self->{ib_xml} ."\n";
		while(my $file = readdir($dh)) {
			chomp($file);
			$file =~ s/^\s*|\s*$//g;
#$file = "" if (($file =~ m/Fri/) && ($reset)); # just for test purposes: don't read in Wednesdays
			if ($file =~ m/[Mon|Tue|Wed|Thu|Fri]\.audit\.xml$/) {
				#print "scanning file '$file'\n";

				my $parser = XML::LibXML->new();
				my $tree = $parser->parse_file($self->{ib_xml} ."/". $file);
				my $root = $tree->getDocumentElement;

				my @nodes = $tree->findnodes('/AuditTrail/Entry/field[@tag="6010"]/..');
				foreach my $node (@nodes) {
					# retrieve the message order type
					# <Entry type="PlaceOrder" msgId="3691650919">
					my $type  = $node->findvalue('@type');
					my $msgid = $node->findvalue('@msgId');

					# retrieve the MultiCharts reference
					# <field tag="6010" val="MultiCharts: ID=[20], Symbol:  (STK, USD, ), Exchange: SMART, &#9;&#9;&#9;   Order: BUY-STP, Lots: 3145, StopPrice: 31.79, LimitPrice: 1.79769e+308"/>
					my $mc = ($node->findnodes('field[@tag="6010"]'))[0]->findvalue('@val');
					$mc =~ s/^MultiCharts.+(Order)/$1/; # remove everything before 'Order'
					$mc =~ s/, LimitPrice: 1.79769e\+308//;
				
					# retrieve the order symbol
					# <field tag="55"   val="MU"/> # symbol
					my $symbol = ($node->findnodes('field[@tag="55"]'))[0]->findvalue('@val');
					
					# retrieve time the order got filled
					# <field tag="52"   val="20150212-14:45:01"/> # time order filled
					my $time = ($node->findnodes('field[@tag="52"]'))[0]->findvalue('@val');

					# retrieve time the order got placed
					# <field tag="6532" val="20150212-14:45:01"/> # time order was placed
					#my $mc = ($node->findnodes('field[@tag="6532"]'))[0]->findvalue('@val');

					# retrieve order size filled and open
					# <field tag="14"   val="500"/> # order size filled
					#my $flots = ($node->findnodes('field[@tag="14" ]'))[0]->findvalue('@val');

					# <field tag="151"  val="2645"/> # order size open
					#my $olots = ($node->findnodes('field[@tag="151"]'))[0]->findvalue('@val');

					my $key = "$time-$type-$msgid";
				
					if (($type eq "PlaceOrder") 
					||  ($type eq "Filled") 
					||  ($type eq "Expired")) {
						#my $msg = "$time, $type\nSymbol: $symbol; $mc";
						my $msg = "$type\nSymbol: $symbol; $mc";

						# retrieve the fill price, if available
						# <field tag="6"    val="31.789618"/>
						if ($node->findnodes('field[@tag="6"]')) {
							my $fprice = ($node->findnodes('field[@tag="6"]'))[0]->findvalue('@val');
							$msg = "$msg, FillPrice: $fprice" if ($fprice > 0);
						}
						$self->{newmsg}{$key} = $msg;
					} elsif (($type eq "ModifyOrder")
					||       ($type eq "PartiallyFilled")
					||       ($type eq "Cancelled")
					||       ($type eq "Acknowledged")) {
						# ModifyOrder:     happens, when trailing stop is adjusted
						# PartiallyFilled: there will also be a message when order is fully filled
						# Cancelled:       MultiCharts doesn't cancel orders
						# Acknowledged:    we assume that an order placed will be acknowledged
					} else {
						print "unknwon type '$type'\n";
					}
				}
			}
		}
	closedir($dh);

	if ($reset) {
		#print "setting current status as baseline\n";
		$self->{oldmsg} = $self->{newmsg};

		foreach my $timestamp (sort(keys(%{$self->{newmsg}}))) {
			print "skipping: ". $self->{newmsg}{$timestamp} ."\n";
		}

		$self->{newmsg} = (); #undef ($self->{newmsg});
	} else {
		#print "calculating delta to baseline\n";
		foreach my $timestamp (keys(%{$self->{newmsg}})) {
			if (exists($self->{oldmsg}{$timestamp})) {
				# this one is already known
				delete $self->{newmsg}{$timestamp};
			} else {
				# this is a new one - also add it to oldmsg
				$self->{oldmsg}{$timestamp} = $self->{newmsg}{$timestamp};
			}
		}
	}
}



#===============================================================================
1;

#===============================================================================
# end of file
